#include "htmlretrieve.hpp"
#include "duckscraperConfig.h"
#include <ciso646>
#include <tidy.h>
#include <buffio.h>
#include <sstream>
#include <curl_easy.h>
#include <cstring>
#include <stack>
#include <algorithm>

namespace duck {
	namespace {
		std::string cleanHTML(const std::string &html) {
			// Initialize a Tidy document
			TidyDoc tidyDoc = tidyCreate();
			TidyBuffer tidyOutputBuffer;
			std::memset(&tidyOutputBuffer, 0, sizeof(TidyBuffer));

			// Configure Tidy
			// The flags tell Tidy to output XML and disable showing warnings
			bool configSuccess = tidyOptSetBool(tidyDoc, TidyXmlOut, yes)
				&& tidyOptSetBool(tidyDoc, TidyQuiet, yes)
				&& tidyOptSetBool(tidyDoc, TidyNumEntities, yes)
				&& tidyOptSetBool(tidyDoc, TidyShowWarnings, no);

			int tidyResponseCode = -1;

			// Parse input
			if (configSuccess)
				tidyResponseCode = tidyParseString(tidyDoc, html.c_str());

			// Process HTML
			if (tidyResponseCode >= 0)
				tidyResponseCode = tidyCleanAndRepair(tidyDoc);

			// Output the HTML to our buffer
			if (tidyResponseCode >= 0)
				tidyResponseCode = tidySaveBuffer(tidyDoc, &tidyOutputBuffer);

			// Any errors from Tidy?
			if (tidyResponseCode < 0)
				throw ("Tidy encountered an error while parsing an HTML response. Tidy response code: " + tidyResponseCode);

			// Grab the result from the buffer and then free Tidy's memory
			std::string tidyResult = (char*)tidyOutputBuffer.bp;
			tidyBufFree(&tidyOutputBuffer);
			tidyRelease(tidyDoc);

			return tidyResult;
		}

		bool isHttps (const std::string& parUrl) {
			const char protocol[] = "https://";
			const size_t protocolLen = sizeof(protocol) / sizeof(protocol[0]) - 1;
			if (parUrl.size() < protocolLen)
				return false;

			return std::equal(protocol, protocol + protocolLen, parUrl.begin());
		}
	} //unnamed namespace

	std::string getCleanHtml (const std::string& parSource, const std::string& parUserAgent, bool parSslVerifyPeer, bool parSslVerifyHost) {
		std::ostringstream oss;
		curl::curl_easy easy(oss);
		easy.add(curl::curl_pair<CURLoption, std::string>(CURLOPT_URL, parSource));
		if (isHttps(parSource)) {
			easy.add(curl::curl_pair<CURLoption, bool>(CURLOPT_SSL_VERIFYPEER, parSslVerifyPeer));
			easy.add(curl::curl_pair<CURLoption, bool>(CURLOPT_SSL_VERIFYHOST, parSslVerifyHost));
		}
		easy.add(curl::curl_pair<CURLoption, std::string>(CURLOPT_USERAGENT, parUserAgent));
		easy.add(curl::curl_pair<CURLoption, long>(CURLOPT_FOLLOWLOCATION, 1L));

		//try {
			easy.perform();
		//}
		//catch (curl_error& err) {
			//std::stack<std::pair<std::string, std::string> > errors = err.what();
			//err.print_traceback();
			//return 1;
		//}
		return cleanHTML(oss.str());
	}

	std::string getCleanHtml (const std::string& parSource, bool parSslVerifyPeer, bool parSslVerifyHost) {
		return getCleanHtml(parSource, std::string(DEFAULT_USER_AGENT), parSslVerifyPeer, parSslVerifyHost);
	}
} //namespace duck
