#include "htmlretrieve.hpp"
#include "duckscraperConfig.h"
#include <iostream>
#include <string>
#include <pugixml.hpp>
#include <sstream>

int main (int argc, char* argv[]) {
	if (argc != 3) {
		std::cerr << PROGRAM_NAME << "v" << VERSION_MAJOR << "." << VERSION_MINOR;
#if VERSION_BETA
		std::cerr << "b";
#endif
		std::cerr << " git revision " << VERSION_GIT << "\n";
		std::cerr << "Default user agent is \"" << DEFAULT_USER_AGENT << "\"\n";
		std::cerr << "Usage: scraper <URL> <XPath>" << std::endl;
		return 2;
	}

	const char* const& url = argv[1];
	const char* const& xpath = argv[2];

	std::cout << "URL  : " << url << "\n";
	std::cout << "XPath: " << xpath << std::endl;

	std::string tidyHtml = duck::getCleanHtml(url, false, false);

	{
		pugi::xml_document doc;
		std::istringstream iss(tidyHtml);
		pugi::xml_parse_result result(doc.load(iss));
		if (not result) {
			std::cerr << "Error parsing the source XML";
			return 1;
		}

		pugi::xpath_node_set xpathRes = doc.select_nodes(xpath);
		for (pugi::xpath_node_set::const_iterator itFind(xpathRes.begin()), itFindEND(xpathRes.end()); itFind != itFindEND; ++itFind) {
			const pugi::xpath_node& node = *itFind;
			if (node.node()) {
				std::cout << node.node().name() << ": " << node.node().value() << "\n";
			}
			else if (node.attribute()) {
				std::cout << node.attribute().name() << ": " << node.attribute().value() << "\n";
			}
		}
	}
	return 0;
}
